package main

import (
	"bufio"
	"flag"
	"fmt"
	"io"
	"os"
)

/*
go run main.go -h
Usage of /var/folders/ql/nw5zyrg905z78mq4kwmq1lg00000gn/T/go-build10851603/b001/exe/main:
  -d string
        destination file path
  -s string
        source file path
*/

func main() {
	src := flag.String("s", "", "source file path")
	dest := flag.String("d", "", "destination file path")
	help := flag.Bool("h", false, "print help")
	flag.Usage = func() {
		fmt.Println(`Usage of copyfile, copy file from one to another.
copyfile -s <src-file> -d <dest-file>
Options:`)
		flag.PrintDefaults()
	}
	flag.Parse()
	if *help || (*src == "" && *dest == "") {
		flag.Usage()
	} else {
		copyfile(*src, *dest)
	}

	//fmt.Printf("%T %T %#v %#v\n", src, *dest, src, *dest)
}

func copyfile(src, dest string) {
	srcFile, err := os.Open(src)
	if err != nil {
		fmt.Printf(err.Error())
		return
	}
	defer srcFile.Close()

	destFile, err := os.Create(dest)
	if err != nil {
		fmt.Println(err.Error())
		return
	}
	defer destFile.Close()

	// 复制策略：1M copy 1次
	bs := make([]byte, 1024*1024)
	reader := bufio.NewReader(srcFile)
	writer := bufio.NewWriter(destFile)
	for {
		//n, err := srcFile.Read(bs)
		n, err := reader.Read(bs)
		if err != nil {
			if err != io.EOF {
				fmt.Println(err.Error())
			}
			break
		} else {
			//destFile.Write(bs[:n])
			nn, err := writer.Write(bs[:n])
			if err != nil {
				fmt.Println(err.Error())
				return
			}
			if n != nn {
				fmt.Printf("read %d bytes, write %d bytes", n, nn)
			}
		}
	}
	err = writer.Flush()
	if err != nil {
		fmt.Println(err.Error())
		return
	}
}
